#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import maya.cmds as cmds


# naming conventions
import hmScripts.hm_global_variables as nc

import hmScripts.Main.hmSelectionHandling as hmSelect
import hmScripts.Main.hmBaseObject as hmNode
import pymel.core as pyc


# ------------------------------------------------------------------------------------------------------------------------
def undo(func):
    def wrapper(*args, **kwargs):
        cmds.undoInfo(openChunk=True)
        try:
            ret = func(*args, **kwargs)
        finally:
            cmds.undoInfo(closeChunk=True)
        return ret
    return wrapper


def create_unique_name(name):
    """
    Get the next non clashing name by adding numbers in the end of the provided name

    :param str name: the input name
    :return str: the unique name
    """
    # If the object exists in the scene,
    # check if is ending with a number.
    if cmds.objExists(name):

        # If it ends with a number, cycle on the reversed
        # word and get the numbers. stop as soon as we get a letter
        if name[-1].isdigit():
            num = ''
            rev_name = name[::-1]
            for x in rev_name:
                if x.isdigit():
                    num += x
                else:
                    break

            # Compose the base name and increment the number
            name = rev_name.replace(num, '', 1)[::-1]
            num = int(num[::-1]) + 1

        else:
            num = 1

        name = '{}{}'.format(name, num)
    else:
        return name

    # Repeat this process iteratively to until we find a unique name
    return create_unique_name(name)

# ------------------------------------------------------------------------------------------------------------------------
@undo
def perform_reference_copy():
    """
    Perform the reference copy from validated selection
    """
    selection = hmSelect.HmSelectionHandling()
    selection.setErrorMessageIfZero('Selection is empty, please select a mesh')
    valid_sel = selection.validate(selType = 'mesh' )

    reference_copy(valid_sel)


def reference_copy(meshes):
    """
    Create a copy of the current mesh by creating a new mesh directly connected to the current mesh.
    :param meshes:
    :return list[HmBaseObject]: list of HmBaseObject of copied meshes
    """
    def reset_edits(in_mesh):
        """
        Reset the edit on the duplicated mesh for avoiding weird behaviors..
        """
        mesh = pyc.PyNode( in_mesh )
        shape = mesh.getShape()
        points = shape.getPoints()

        for id, point in enumerate(points):
            for ax in ['x', 'y', 'z']:
                shape.setAttr('pnts[{}].pnt{}'.format(id, ax), 0 )

    out_meshes = []

    for el in meshes:

        orig_obj = hmNode.HmBaseObject(el)
        duplicated = clean_duplicate([el])[0]
        orig_obj.get_shape().connect_attribute('outMesh', duplicated.get_shape(), 'inMesh')

        reset_edits(duplicated)

        # If we can't parent to world, simply do not raise the error
        #
        try:
            cmds.parent(duplicated.get_name(), world=True)
        except:
            pass
        finally:
            start_name = validate_name(orig_obj.get_name())
            new_name = '{}_{}'.format(nc.reference_copy, start_name)
            # print new_name
            duplicated.set_name( new_name )

            out_meshes.append(duplicated)

    return out_meshes

@undo
def clean_duplicate_input():
    '''
    Duplicate the selected hm_in_mesh and remove the attributes
    '''
    selection = hmSelect.HmSelectionHandling()
    selection.setErrorMessageIfZero('Selection is empty, please select an Hard Mesh input to clean duplicate.')
    valid_sel = selection.validate(selType = nc.hm_in_mesh )


    return clean_duplicate(valid_sel)

@undo
def clean_duplicate_output():
    '''
    Duplicate the selected hm_out_mesh and remove the attributes
    '''
    selection = hmSelect.HmSelectionHandling()
    selection.setErrorMessageIfZero('Selection is empty, please select an Hard Mesh output to clean duplicate.')
    valid_sel = selection.validate(selType = nc.hm_out_mesh )

    return clean_duplicate(valid_sel)

@undo
def clean_duplicate(valid_sel):
    '''
    Basic function that deletes attributes from the mesh

    Args:
        valid_sel: list[strings]: list of valid hard mesh / maya objects
    '''
    out = []
    out_selection = []

    for mesh in valid_sel:

        # Duplicate the current mesh and init as HmObject
        #
        duplicated = cmds.duplicate(mesh)[0]
        hm_object = hmNode.HmBaseObject( duplicated )

        # Clean from useless stuff and add to the return
        #
        clean_object( hm_object )

        out.append(hm_object)
        out_selection.append(hm_object.get_name())

    cmds.select(out_selection, r=True)

    return out

@undo
def clean_object(mesh):
    '''
    Clean an input mesh from the usless connections and attributes

    Args:
        mesh: HmBaseObj of a mesh to be cleaned

    Returns:
        None
    '''
    # print 'cleaning object %s from hard mesh attributes and connections' % mesh.get_name()

    mesh.remove_hm_attributes()
    mesh.reset_display_overrides()

    # Delete the poly smooth, inter mesh and the offset
    shape = mesh.get_shape()

    # Is there a smooth connected?
    smooth = shape.get_connected('outMesh', 'out', node_type = 'polySmoothFace')

    # If yes, continue the research of an intermediate mesh
    if smooth:
        smooth_node = smooth[0][0]

        interm = smooth_node.get_connected('output', 'out')

        # Delete poly smooth
        # print 'lo smooth :', smooth_node.get_name()
        smooth_node.delete()

        # Is there the intermediate mesh?
        if interm:

            interm_mesh = interm[0][0]

            offset = interm_mesh.get_connected('outMesh', 'out', node_type = 'hmOffsetCurveNode')

            # Delete intermediate mesh
            # print 'l interm mesh :', interm_mesh.get_name()
            interm_mesh.delete()

            # Is there an offset node?
            if offset:
                offset_node = offset[0][0]

                # Delete offset
                # print 'l offset :', offset_node.get_name()
                offset_node.delete()

# ------------------------------------------------------------------------------------------------------------------------
def validate_name( name):
    '''
    Validate the name from possible garbage

    Args:
        input: a string rapresenting a maya object name
    '''
    if '|' in name:
        return name.split('|')[-1]
    else:
        return name


def getPathFromLongName(stringLongName ):
    list = stringLongName.split( '|' ) #splittiamo..
    list.remove( list[len(list) -1] ) #togliamo il nome finale..
    path = '|'.join(list) + '|' #ricomponiamo una stringa senza il nome finale..
    return path

def hasNumbers(inputString):
    '''
    Query if the string contains numbers and.. return them?
    '''
    return any(char.isdigit() for char in inputString)

def getNumbers(inputString):
    #return [int(s) for s in inputString.split() if s.isdigit()] #funziona solo se il numero ha uno spazio.
    return re.findall('\d+', inputString)



def flatten(x):
    '''
    flatten(sequence) -> list

    Returns a single, flat list which contains all elements retrieved
    from the sequence and all recursively contained sub-sequences
    (iterables).

    Examples:
    >>> [1, 2, [3,4], (5,6)]
    [1, 2, [3, 4], (5, 6)]
    >>> flatten([[[1,2,3], (42,None)], [4,5], [6], 7, MyVector(8,9,10)])
    [1, 2, 3, 42, None, 4, 5, 6, 7, 8, 9, 10]
    '''
    result = []
    for el in x:
        #if isinstance(el, (list, tuple)):
        if hasattr(el, "__iter__") and not isinstance(el, basestring):
            result.extend(flatten(el))
        else:
            result.append(el)
    return result
